/* globals DOpus, ActiveXObject */
/* jshint -W027 */
function OnInit(initData) {
	initData.name = "SaveTabGroups";
	initData.desc = "SaveTabGroups";
	initData.copyright = "(c) 2020 Benjamin Philipp";
	initData.default_enable = true;
	initData.version = "1.0.1";
	initData.log_prefix = initData.name;
}

var TEST_directSync = true;
var saveAsName = "AutoSave";
//var groupsLocation = "%APPDATA%\\GPSoftware\\Directory Opus\\TabGroups";
var groupsLocation = "/dopusdata\\TabGroups";
var pathBase = groupsLocation + "\\" + saveAsName;
var maxBackups = 10;

function OnOpenLister(data){
	if(data.after === false){
		sglob("autoSaveEnabled", false);
	}
	else{
		sglob("autoSaveEnabled", true);
	}
	return true;
}

function OnAfterFolderChange(data){
//	log(data.action);
	saveGroup();
}

function OnOpenTab(data){
	saveGroup();
}

function OnCloseTab(data){
	saveGroup();
}

function saveGroup(){
	if(TEST_directSync){ // Don't async run the internal command "SAVETABS" but do it right here
		saveTabs();
		return;
	}
	
//	log("Run internal command async?");
	var cmd = DOpus.Create().Command;
	cmd.AddLine("@noprogress");
	cmd.AddLine("@runmode:hide");
//	cmd.AddLine("/dopusdata\\Script AddIns\\saveTabs.bat");
//	cmd.AddLine("SAVETABS");
	cmd.AddLine("@async:dopusrt /cmd SAVETABS");
	cmd.Run();
}

function saveTabs(){ // Basically what the internal command "SAVETABS" does
	if(glob("autoSaveEnabled") !== true)
		return;
	
	log("Saving tabs");
	
	var cmd = DOpus.Create().Command;
//	cmd.AddLine("@noprogress");
//	cmd.AddLine("@runmode:hide");
	
//	cmd.AddLine("DELETE FILE=\"" + pathBase + " - Backup " + maxBackups + ".otg\" FORCE QUIET RECYCLE");
	for(var i = maxBackups-1; i>=0; i--){
		var cf = pathBase + " - Backup " + i + ".otg";
		if(i===0){
			cf = pathBase + ".otg";
		}
		if(DOpus.FSUtil.Exists(cf))
			cmd.RunCommand("RENAME FROM=\"" + cf + "\" TO=\"" + saveAsName + " - Backup " + (i+1) + ".otg\" NOMATCHNOFAIL TYPE=files WHENEXISTS=delete");
//			cmd.AddLine("RENAME FROM=\"" + cf + "\" TO=\"" + saveAsName + " - Backup " + (i+1) + ".otg\" NOMATCHNOFAIL TYPE=files WHENEXISTS=delete");
	}
	cmd.RunCommand("GO TABGROUPSAVE \"" + saveAsName + "\"");
//	cmd.AddLine("GO TABGROUPSAVE \"" + saveAsName + "\"");
//	cmd.Run();
}

//////////////////////// only helper functions below ///////////////////////////////////

function stringify(obj){
	if(obj===undefined)
		obj = "<undefined>";
	else if(obj === null)
		obj = "<null>";
	else if(typeof obj == "object"){
		var o = obj;
		obj = "[Object]\n";
		obj += dump(o);
	}
	else if(obj === false)
		obj = "FALSE";
	else if(obj === true)
		obj = "TRUE";
	else if(obj==="")
		obj = "<empty>";
	return obj;
}

function msg(s){
	var dlg = DOpus.Dlg;
	s = stringify(s);
	log("MSG: " + s);
	dlg.message = s;
	dlg.buttons = "OK";
	dlg.icon = "info";
	var i = dlg.Show();
}

function dump(arr, level){
	var dumped_text = "";
	if (!level) level = 0;
	
	var bs = "\n";

	var level_padding = "	";
	for (var j = 0; j < level + 1; j++) level_padding += "	 ";

	if (typeof (arr) == 'object'){ //Array/Hashes/Objects 
		for (var item in arr){
			var value = arr[item];

			if (typeof (value) == 'object'){ //If it is an array,
				dumped_text += level_padding + "'" + item + "'(" + typeof (value) + "){" + bs;
				dumped_text += dump(value, level + 1) + bs + "}";
			} else {
				dumped_text += level_padding + "'" + item + "' => \"" + value + "\"" + bs;
			}
		}
	} else { //Stings/Chars/Numbers etc.
		dumped_text = "===>" + arr + "<===(" + typeof (arr) + ")";
	}
	return dumped_text;
}

function log(obj){
	DOpus.Output(stringify(obj));
}

function trim(str){
	var rex = /^(?:\s)*((?:[\r\n]|.)*?)(?:\s)*$/;
	var m = str.match(rex);
	if(m===null)
		return "";
	if(m.length>=1)
		return m[1];
	return "";
}

String.prototype.trim = function(){
	return trim(this);
};
	
String.prototype.appendLine = function(l){
	var r = this;
	if(r === ""){
		r = "" + l;
	}
	else{
		r += "\r\n" + l;
	}
	return r;
};

String.prototype.after = function(str, fromRight, returnAll){
	if(fromRight === undefined)
		fromRight = false;
	if(returnAll === undefined)
		returnAll = false;
	var os = this.indexOf(str);
	if(fromRight)
		os = this.lastIndexOf(str);
	if(os<0)
		return returnAll?this:"";
	return this.substr(os + str.length);
};
 
String.prototype.before = function(str, fromRight, returnAll){
	if(fromRight === undefined)
		fromRight = false;
	if(returnAll === undefined)
		returnAll = false;
	var os = this.indexOf(str);
	if(fromRight)
		os = this.lastIndexOf(str);
	if(os<0)
		return returnAll?this:"";
	return this.substr(0, os);
};

String.prototype.basename = function(){
	return this.after("\\", true, true).before(".", true, true);
};

Array.prototype.toVector = function(){
	var ret = DOpus.Create.Vector(this.length);
	for(var i=0; i<this.length;i++){
		var a = this[i];
		ret(i) = a; // jshint ignore:line
	}
	return ret;
};

function writeToFile(file, str, Encoding) {
	var encoding = Encoding!==undefined?Encoding : "UTF-8"; // "windows-1252"; //
	
	var UTFStream = new ActiveXObject('ADODB.Stream');
	UTFStream.Type = 2;
	UTFStream.Charset = encoding;
	UTFStream.Open();
	UTFStream.WriteText(str);
	UTFStream.SaveToFile(file, 2);
	UTFStream.Flush();
	UTFStream.Close();
}

function readFromFile(file, silent) {
	if(silent===undefined)
		silent = false;
	file = expand(file);
	var fso = new ActiveXObject("Scripting.FileSystemObject");
	try{
		var fileStream = fso.openTextFile(file);
		var fileData = fileStream.readAll();
		fileStream.Close();
		return fileData;
	}
	catch(e){
		if(!silent)
			msg("Could not open file '" + file + "'\nError: " + e);
	}
	return "";
}

function expand(path){
	var wshShell = new ActiveXObject("WScript.Shell");
	return wshShell.ExpandEnvironmentStrings(path);
}

function lz(num, places){
	if(!places && places !== 0)
		places=2;
	return ("000000000000000" + num).slice(-places);
}

Array.prototype.includes = function(valueToFind, fromIndex){

	if (this === null) {
		return false;
	}

	var o = Object(this);
	var len = o.length >>> 0;

	if (len === 0)
		return false;

	var n = fromIndex | 0;
	var k = Math.max(n >= 0 ? n : len - Math.abs(n), 0);

	function sameValueZero(x, y) {
		return x === y || (typeof x === 'number' && typeof y === 'number' && isNaN(x) && isNaN(y));
	}

	while (k < len) {
		if (sameValueZero(o[k], valueToFind)) {
			return true;
		}
		k++;
	}
	return false;
};

function glob(varname, defaultReturn, defaultOnEmpty){
	defaultReturn = defaultReturn!==undefined?defaultReturn:"";
	defaultOnEmpty = defaultOnEmpty!==undefined?defaultOnEmpty:false;
	var ret=null;
	try{
		ret = DOpus.vars.Get(varname);
	}catch(e){
//		log(e);
	}
	if(ret===undefined || (defaultOnEmpty && (ret==="" || ret===null))){
		sglob(varname, defaultReturn);
		return defaultReturn;
	}
	return ret;
}

function sglob(varname, value){
	DOpus.vars.set(varname, value);
}